/* tapfeszultseg_meghatarozasa
 *
 * A belső 1,1 V-os referencia segítségével meghatározzuk
 * a névlegesen 5 V-os tápfeszültség tényleges értékét.
 * Mivel a belső referenciával nem tudunk 1,1 V-nál nagyobb
 * feszültséget megmérni, így fordítva járunk el: az 0b1110
 * csatornára kötött belső referenciát mérjük meg VCC 
 * referenciával és abból számolunk visszafelé, az alábbi
 * aránypár szerint:
 *
 *  1024 : Nadc = Vcc : Vref  => Vcc = Vref * 1024/Nadc
 * 
 * ahol Vref = 1100 mV és Nadc az ADC-ből kapott szám
 *
 * Források: http://www.tavir.hu/konyv-trukkok-internal
 * valamint https://github.com/chatelao/MiniPirate
 */

long readVcc() {
  long result;
  // 1.1V referenciafeszültség mérése, AVcc a referencia
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  delay(20);                        // Vref beállási idő
  ADCSRA |= _BV(ADSC);              // Konverzió indítása
  while (bit_is_set(ADCSRA,ADSC));  // Konverzió végére vár
  result = ADCL;                    // Előbb ezt kell kiolvasni
  result |= ADCH<<8;                // Magasabb helyiértékű bitek 
  result = 1100L * 1024L / result;  // AVcc kiszámítása [mV]
  return result;                    // 1100 mV Vref * 1024 ADC felbontás
}

void setup() {
  Serial.begin(9600);
  Serial.println("Tápfeszültség meghatározása belső referenciával");
}

void loop() {
  long vcc = readVcc();
  Serial.print("Vcc = ");
  Serial.print(vcc);
  Serial.println(" mV");
  delay(5000);
}
